@@ -32,6 +32,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 #include <ctype.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -260,8 +264,7 @@ int mutt_edit_attachment (BODY *a)
   else if (a->type == TYPETEXT)
   {
     /* On text, default to editor */
-    mutt_edit_file ((!Editor || mutt_strcmp ("builtin", Editor) == 0) ? 
-		    NONULL(Visual) : NONULL(Editor), newfile);
+    mutt_edit_file (NONULL (Editor), newfile);
   }
   else
   {
@@ -325,6 +328,41 @@ static int is_mmnoask (const char *buf)
   return (0);
 }
 
+void mutt_check_lookup_list (BODY *b, char *type, int len)
+{
+  LIST *t = MimeLookupList;
+  int i;
+
+  for (; t; t = t->next) {
+    i = mutt_strlen (t->data) - 1;
+    if ((i > 0 && t->data[i-1] == '/' && t->data[i] == '*' && 
+	 ascii_strncasecmp (type, t->data, i) == 0) ||
+	ascii_strcasecmp (type, t->data) == 0) {
+
+    BODY tmp = {0};
+    int n;
+    if ((n = mutt_lookup_mime_type (&tmp, b->filename)) != TYPEOTHER) {
+      snprintf (type, len, "%s/%s",
+                n == TYPEAUDIO ? "audio" :
+                n == TYPEAPPLICATION ? "application" :
+                n == TYPEIMAGE ? "image" :
+                n == TYPEMESSAGE ? "message" :
+                n == TYPEMODEL ? "model" :
+                n == TYPEMULTIPART ? "multipart" :
+                n == TYPETEXT ? "text" :
+                n == TYPEVIDEO ? "video" : "other",
+                tmp.subtype);
+      dprint(1, (debugfile, "mutt_check_lookup_list: \"%s\" -> %s\n", 
+        b->filename, type));
+    }
+    if (tmp.subtype) 
+      safe_free ((void **) &tmp.subtype);
+    if (tmp.xtype) 
+      safe_free ((void **) &tmp.xtype);
+    }
+  }
+}
+
 int mutt_is_autoview (BODY *b, const char *type)
 {
   LIST *t = AutoViewList;
@@ -332,10 +370,12 @@ int mutt_is_autoview (BODY *b, const char *type)
   int i;
 
   if (!type)
-  {
     snprintf (_type, sizeof (_type), "%s/%s", TYPE (b), b->subtype);
-    type = _type;
-  }
+  else
+    strncpy (_type, type, sizeof(_type));
+
+  mutt_check_lookup_list (b, _type, sizeof(_type));
+  type = _type;
 
   if (mutt_needs_mailcap (b))
   {
@@ -346,8 +386,7 @@ int mutt_is_autoview (BODY *b, const char *type)
       return 1;
   }
 
-  for (; t; t = t->next)
-  {
+  for (; t; t = t->next) {
     i = mutt_strlen (t->data) - 1;
     if ((i > 0 && t->data[i-1] == '/' && t->data[i] == '*' && 
 	 ascii_strncasecmp (type, t->data, i) == 0) ||
@@ -377,10 +416,11 @@ int mutt_view_attachment (FILE *fp, BODY *a, int flag, HEADER *hdr,
   int unlink_tempfile = 0;
   
   is_message = mutt_is_message_type(a->type, a->subtype);
-#ifdef HAVE_PGP
-  if (is_message && a->hdr && (a->hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (is_message && a->hdr && (a->hdr->security & ENCRYPT) &&
+      !crypt_valid_passphrase(a->hdr->security))
     return (rc);
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
   use_mailcap = (flag == M_MAILCAP ||
 		(flag == M_REGULAR && mutt_needs_mailcap (a)));
   snprintf (type, sizeof (type), "%s/%s", TYPE (a), a->subtype);
@@ -675,7 +715,13 @@ bail:
   if (outfile && *outfile)
     close (out);
 
-  if (rv == 0 || mutt_wait_filter (thepid) != 0 || option (OPTWAITKEY))
+  /*
+   * check for error exit from child process
+   */
+  if (mutt_wait_filter (thepid) != 0)
+    rv = 0;
+
+  if (rv == 0 || option (OPTWAITKEY))
     mutt_any_key_to_continue (NULL);
   return rv;
 }
