@@ -21,6 +21,8 @@
 #include "mailbox.h"
 #include "mx.h"
 
+#include "mutt_curses.h"
+
 #ifdef USE_IMAP
 #include "imap.h"
 #endif
@@ -162,7 +164,7 @@ void mutt_update_mailbox (BUFFY * b)
 
 int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  BUFFY **tmp;
+  BUFFY **tmp,*tmp1;
   char buf[_POSIX_PATH_MAX];
 #ifdef BUFFY_SIZE
   struct stat sb;
@@ -172,6 +174,19 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *e
   {
     mutt_extract_token (path, s, 0);
     strfcpy (buf, path->data, sizeof (buf));
+
+    if(data == M_UNMAILBOXES && mutt_strcmp(buf,"*") == 0)
+    {
+      for (tmp = &Incoming; *tmp;)
+      {
+        safe_free((void **)&((*tmp)->path));
+        tmp1=(*tmp)->next;
+        safe_free((void **)tmp);
+        *tmp=tmp1;
+      }
+      return 0;
+    }
+
     mutt_expand_path (buf, sizeof (buf));
 
     /* Skip empty tokens. */
@@ -184,6 +199,18 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *e
 	break;
     }
 
+    if(data == M_UNMAILBOXES)
+    {
+      if(*tmp)
+      {
+        safe_free((void **)&((*tmp)->path));
+        tmp1=(*tmp)->next;
+        safe_free((void **)tmp);
+        *tmp=tmp1;
+      }
+      continue;
+    }
+
     if (!*tmp)
     {
       *tmp = (BUFFY *) safe_calloc (1, sizeof (BUFFY));
@@ -397,29 +424,66 @@ int mutt_buffy_check (int force)
   return (BuffyCount);
 }
 
-int mutt_buffy_notify (void)
+int mutt_buffy_list (void)
 {
   BUFFY *tmp;
   char path[_POSIX_PATH_MAX];
+  char buffylist[160];
+  int pos;
+  int first;
 
+  int have_unnotified = BuffyNotify;
+  
+  pos = 0;
+  first = 1;
+  buffylist[0] = 0;
+  pos += strlen (strncat (buffylist, _("New mail in "), sizeof (buffylist) - 1 - pos));
+  for (tmp = Incoming; tmp; tmp = tmp->next)
+  {
+    /* Is there new mail in this mailbox? */
+    if (!tmp->new || (have_unnotified && tmp->notified))
+      continue;
+
+    strfcpy (path, tmp->path, sizeof (path));
+    mutt_pretty_mailbox (path);
+    
+    if (!first && pos + strlen (path) >= COLS - 7)
+      break;
+    
+    if (!first)
+      pos += strlen (strncat(buffylist + pos, ", ", sizeof(buffylist)-1-pos));
+
+    /* Prepend an asterisk to mailboxes not already notified */
+    if (!tmp->notified)
+    {
+      /* pos += strlen (strncat(buffylist + pos, "*", sizeof(buffylist)-1-pos)); */
+      tmp->notified = 1;
+      BuffyNotify--;
+    }
+    pos += strlen (strncat(buffylist + pos, path, sizeof(buffylist)-1-pos));
+    first = 0;
+  }
+  if (!first && tmp)
+  {
+    strncat (buffylist + pos, ", ...", sizeof (buffylist) - 1 - pos);
+  }
+  if (!first)
+  {
+    mutt_message ("%s", buffylist);
+    return (1);
+  }
+  /* there were no mailboxes needing to be notified, so clean up since 
+   * BuffyNotify has somehow gotten out of sync
+   */
+  BuffyNotify = 0;
+  return (0);
+}
+
+int mutt_buffy_notify (void)
+{
   if (mutt_buffy_check (0) && BuffyNotify)
   {
-    for (tmp = Incoming; tmp; tmp = tmp->next)
-    {
-      if (tmp->new && !tmp->notified)
-      {
-	strfcpy (path, tmp->path, sizeof (path));
-	mutt_pretty_mailbox (path);
-	mutt_message (_("New mail in %s."), path);
-	tmp->notified = 1;
-	BuffyNotify--;
-	return (1);
-      }
-    }
-    /* there were no mailboxes needing to be notified, so clean up since 
-     * BuffyNotify has somehow gottten out of sync
-     */
-    BuffyNotify = 0;
+    return (mutt_buffy_list ());
   }
   return (0);
 }
