@@ -13,9 +13,13 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
+/*parameter to mutt_parse_mailboxes*/
+#define M_MAILBOXES   1
+#define M_UNMAILBOXES 2 
+
 typedef struct buffy_t
 {
   char *path;
