@@ -28,8 +28,9 @@ typedef void *iconv_t;
 #endif
 
 #ifndef HAVE_ICONV
+#define ICONV_CONST /**/
 iconv_t iconv_open (const char *, const char *);
-size_t iconv (iconv_t, const char **, size_t *, char **, size_t *);
+size_t iconv (iconv_t, ICONV_CONST char **, size_t *, char **, size_t *);
 int iconv_close (iconv_t);
 #endif
 
@@ -42,11 +43,14 @@ typedef void * FGETCONV;
 
 FGETCONV *fgetconv_open (FILE *, const char *, const char *, int);
 int fgetconv (FGETCONV *);
+char * fgetconvs (char *, size_t, FGETCONV *);
 void fgetconv_close (FGETCONV **);
 
 void mutt_set_langinfo_charset (void);
 
 #define M_ICONV_HOOK_FROM 1
+#if 0
 #define M_ICONV_HOOK_TO   2
+#endif
 
 #endif /* _CHARSET_H */
