@@ -45,6 +45,11 @@
 #endif
 
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
+
 
 #include <errno.h>
 #include <unistd.h>
@@ -76,18 +81,22 @@ int mutt_display_message (HEADER *cur)
   mutt_parse_mime_message (Context, cur);
   mutt_message_hook (Context, cur, M_MESSAGEHOOK);
 
-#ifdef HAVE_PGP
+#if defined (HAVE_PGP) || defined (HAVE_SMIME)
   /* see if PGP is needed for this message.  if so, we should exit curses */
-  if (cur->pgp)
+  if (cur->security)
   {
-    if (cur->pgp & PGPENCRYPT)
+    if (cur->security & ENCRYPT)
     {
-      if (!pgp_valid_passphrase ())
+#ifdef HAVE_SMIME
+      if (cur->security & APPLICATION_SMIME)
+	smime_getkeys (cur->env);
+#endif
+      if(!crypt_valid_passphrase(cur->security))
 	return 0;
 
       cmflags |= M_CM_VERIFY;
     }
-    else if (cur->pgp & PGPSIGN)
+    else if (cur->security & SIGN)
     {
       /* find out whether or not the verify signature */
       if (query_quadoption (OPT_VERIFYSIG, _("Verify PGP signature?")) == M_YES)
@@ -97,16 +106,26 @@ int mutt_display_message (HEADER *cur)
     }
   }
   
-  if ((cmflags & M_CM_VERIFY) || (cur->pgp & PGPENCRYPT))
+  if (cmflags & M_CM_VERIFY || cur->security & ENCRYPT)
   {
-    if (cur->env->from)
-      pgp_invoke_getkeys (cur->env->from);
-
-    mutt_message _("Invoking PGP...");
-  }
+#ifdef HAVE_PGP
+    if (cur->security & APPLICATION_PGP)
+    {
+      if (cur->env->from)
+        pgp_invoke_getkeys (cur->env->from);
 
+      mutt_message _("Invoking PGP...");
+    }
 #endif
 
+#ifdef HAVE_SMIME
+    if (cur->security & APPLICATION_SMIME)
+      mutt_message _("Invoking OpenSSL...");
+#endif
+  }
+#endif /*  defined (HAVE_PGP) || defined (HAVE_SMIME) */
+
+
   mutt_mktemp (tempfile);
   if ((fpout = safe_fopen (tempfile, "w")) == NULL)
   {
@@ -141,11 +160,14 @@ int mutt_display_message (HEADER *cur)
 
   res = mutt_copy_message (fpout, Context, cur, cmflags,
        	(option (OPTWEED) ? (CH_WEED | CH_REORDER) : 0) | CH_DECODE | CH_FROM);
-  if ((fclose (fpout) != 0 && errno != EPIPE) || res == -1)
+  if ((safe_fclose (&fpout) != 0 && errno != EPIPE) || res == -1)
   {
     mutt_error (_("Could not copy message"));
     if (fpfilterout != NULL)
+    {
       mutt_wait_filter (filterpid);
+      safe_fclose (&fpfilterout);
+    }
     mutt_unlink (tempfile);
     return 0;
   }
@@ -153,18 +175,40 @@ int mutt_display_message (HEADER *cur)
   if (fpfilterout != NULL && mutt_wait_filter (filterpid) != 0)
     mutt_any_key_to_continue (NULL);
 
-#ifdef HAVE_PGP
-  /* update PGP information for this message */
-  cur->pgp |= pgp_query (cur->content);
+  safe_fclose (&fpfilterout);	/* XXX - check result? */
+
+  
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  /* update crypto information for this message */
+  cur->security |= crypt_query (cur->content);
+  
+  /* Remove color cache for this message, in case there
+     are color patterns for both ~g and ~V */
+  cur->pair = 0;
 #endif
 
   if (builtin)
   {
     pager_t info;
+#ifdef HAVE_SMIME
+
+    if (cur->security & APPLICATION_SMIME && (cmflags & M_CM_VERIFY))
+    {
+      if (cur->security & GOODSIGN)
+      {
+	if (!smime_verify_sender(cur))
+	  mutt_message ( _("S/MIME signature successfully verified."));
+	else
+	  mutt_error ( _("S/MIME certificate owner does not match sender."));
+      }
+      else if (cur->security & SIGN || cur->security & BADSIGN)
+	mutt_error ( _("S/MIME signature could NOT be verified."));
+    }
+#endif
 
 #ifdef HAVE_PGP
-    if (cmflags & M_CM_VERIFY)
-      mutt_message ((cur->pgp & PGPGOODSIGN) ?
+    if ((cur->security & APPLICATION_PGP) && (cmflags & M_CM_VERIFY))
+      mutt_message ((cur->security & GOODSIGN) ?
 		    _("PGP signature successfully verified.") :
 		    _("PGP signature could NOT be verified."));
 #endif
@@ -251,9 +295,11 @@ void ci_bounce_message (HEADER *h, int *redraw)
     return;
   }
 
-  mutt_bounce_message (NULL, h, adr);
+  rc = mutt_bounce_message (NULL, h, adr);
   rfc822_free_address (&adr);
-  mutt_message (h ? _("Message bounced.") : _("Messages bounced."));
+  /* If no error, or background, display message. */
+  if ((rc == 0) || (rc == S_BKG))
+    mutt_message (h ? _("Message bounced.") : _("Messages bounced."));
 }
 
 static void pipe_set_flags (int decode, int print, int *cmflags, int *chflags)
@@ -282,17 +328,16 @@ void pipe_msg (HEADER *h, FILE *fp, int decode, int print)
   
   pipe_set_flags (decode, print, &cmflags, &chflags);
 
-#ifdef HAVE_PGP
-  
-  if (decode && (h->pgp & PGPENCRYPT))
-  {
-    if (!pgp_valid_passphrase())
-      return;
-    endwin();
-  }
-  
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+    if (decode && h->security & ENCRYPT)
+    {
+      if(!crypt_valid_passphrase(h->security))
+	return;
+      endwin ();
+    }
 #endif
 
+
   if (decode)
     mutt_parse_mime_message (Context, h);
 
@@ -313,17 +358,22 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
   pid_t thepid;
   FILE *fpout;
   
-  mutt_endwin (NULL);
+/*   mutt_endwin (NULL); 
+
+     is this really needed here ? 
+     it makes the screen flicker on pgp and s/mime messages,
+     before asking for a passphrase...
+                                     Oliver Ehli */
   if (h)
   {
 
     mutt_message_hook (Context, h, M_MESSAGEHOOK);
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     if (decode)
     {
       mutt_parse_mime_message (Context, h);
-      if(h->pgp & PGPENCRYPT && !pgp_valid_passphrase())
+      if(h->security & ENCRYPT && !crypt_valid_passphrase(h->security))
 	return 1;
     }
     mutt_endwin (NULL);
@@ -336,7 +386,7 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
     }
       
     pipe_msg (h, fpout, decode, print);
-    safe_fclose (&fpout);
+    fclose (fpout);
     rc = mutt_wait_filter (thepid);
   }
   else
@@ -344,7 +394,7 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 
     if (decode)
     {
@@ -353,8 +403,8 @@ static int _mutt_pipe_message (HEADER *h, char *cmd,
 	{
 	  mutt_message_hook (Context, Context->hdrs[Context->v2r[i]], M_MESSAGEHOOK);
 	  mutt_parse_mime_message(Context, Context->hdrs[Context->v2r[i]]);
-	  if (Context->hdrs[Context->v2r[i]]->pgp & PGPENCRYPT &&
-	      !pgp_valid_passphrase())
+	  if (Context->hdrs[Context->v2r[i]]->security & ENCRYPT &&
+	      !crypt_valid_passphrase(Context->hdrs[Context->v2r[i]]->security))
 	    return 1;
 	}
     }
@@ -576,18 +626,30 @@ static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags,
   *cmflags = 0;
   *chflags = CH_UPDATE_LEN;
   
-#ifdef HAVE_PGP
-  if (!decode && decrypt && (hdr->pgp & PGPENCRYPT))
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (!decode && decrypt && (hdr->security & ENCRYPT))
   {
+#ifdef HAVE_PGP
     if (mutt_is_multipart_encrypted(hdr->content))
     {
       *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
       *cmflags = M_CM_DECODE_PGP;
     }
-    else if (mutt_is_application_pgp(hdr->content) & PGPENCRYPT)
+    else if (mutt_is_application_pgp (hdr->content) & ENCRYPT)
       decode = 1;
-  }
 #endif
+#if defined(HAVE_PGP) && defined(HAVE_SMIME)
+    else
+#endif
+#ifdef HAVE_SMIME
+	  if (mutt_is_application_smime(hdr->content) & ENCRYPT)
+    {
+      *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
+      *cmflags = M_CM_DECODE_SMIME;
+    }
+#endif
+  }
+#endif /* defined(HAVE_PGP) || defined(HAVE_SMIME) */
 
   if (decode)
   {
@@ -628,8 +690,8 @@ int mutt_save_message (HEADER *h, int delete,
 		       int decode, int decrypt, int *redraw)
 {
   int i, need_buffy_cleanup;
-#ifdef HAVE_PGP
-  int need_passphrase = 0;
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  int need_passphrase = 0, app=0;
 #endif
   char prompt[SHORT_STRING], buf[_POSIX_PATH_MAX];
   CONTEXT ctx;
@@ -651,10 +713,12 @@ int mutt_save_message (HEADER *h, int delete,
 	     (delete ? _("Save%s to mailbox") : _("Copy%s to mailbox"))),
 	    h ? "" : _(" tagged"));
   
+
   if (h)
   {
-#ifdef HAVE_PGP
-    need_passphrase = h->pgp & PGPENCRYPT;
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+      need_passphrase = h->security & ENCRYPT;
+      app = h->security;
 #endif
     mutt_message_hook (Context, h, M_MESSAGEHOOK);
     mutt_default_save (buf, sizeof (buf), h);
@@ -672,12 +736,14 @@ int mutt_save_message (HEADER *h, int delete,
       }
     }
 
+
     if (h)
     {
       mutt_message_hook (Context, h, M_MESSAGEHOOK);
       mutt_default_save (buf, sizeof (buf), h);
-#ifdef HAVE_PGP
-      need_passphrase |= h->pgp & PGPENCRYPT;
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+      need_passphrase = h->security & ENCRYPT;
+      app = h->security;
 #endif
       h = NULL;
     }
@@ -712,8 +778,8 @@ int mutt_save_message (HEADER *h, int delete,
   if (!mutt_save_confirm (buf, &st))
     return -1;
 
-#ifdef HAVE_PGP
-  if(need_passphrase && (decode || decrypt) && !pgp_valid_passphrase())
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (need_passphrase && (decode || decrypt) && !crypt_valid_passphrase(app))
     return -1;
 #endif
   
@@ -865,20 +931,19 @@ void mutt_edit_content_type (HEADER *h, BODY *b, FILE *fp)
   if (fp && (is_multipart (b) || mutt_is_message_type (b->type, b->subtype)))
     mutt_parse_part (fp, b);
   
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
   if (h)
   {
     if (h->content == b)
-      h->pgp = 0;
-    h->pgp |= pgp_query (b);
-  }
-#endif /* HAVE_PGP */
+      h->security  = 0;
 
+    h->security |= crypt_query (b);
+  }
+#endif
 }
 
 
 #ifdef HAVE_PGP
-
 static int _mutt_check_traditional_pgp (HEADER *h, int *redraw)
 {
   MESSAGE *msg;
@@ -889,7 +954,7 @@ static int _mutt_check_traditional_pgp (HEADER *h, int *redraw)
     return 0;
   if (pgp_check_traditional (msg->fp, h->content, 0))
   {
-    h->pgp = pgp_query (h->content);
+    h->security = crypt_query (h->content);
     *redraw |= REDRAW_FULL;
     rv = 1;
   }
