@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -80,6 +84,10 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 	    continue;
 	  from = 1;
 	}
+	else if (flags & (CH_NOQFROM) &&
+			ascii_strncasecmp (">From ", buf, 6) == 0)
+		continue;
+
 	else if (buf[0] == '\n' || (buf[0] == '\r' && buf[1] == '\n'))
 	  break; /* end of header */
 
@@ -276,6 +284,7 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
  	CH_UPDATE_LEN	write new Content-Length: and Lines:
  	CH_XMIT		ignore Lines: and Content-Length:
  	CH_WEED		do header weeding
+	CH_NOQFROM      ignore ">From " line
 
    prefix
    	string to use if CH_PREFIX is set
@@ -417,6 +426,7 @@ static int count_delete_lines (FILE *fp, BODY *b, long *length, size_t datelen)
  * 	M_CM_PREFIX	quote header and body
  *	M_CM_DECODE	decode message body to text/plain
  *	M_CM_DISPLAY	displaying output to the user
+ *      M_CM_PRINTING   printing the message
  *	M_CM_UPDATE	update structures in memory after syncing
  *	M_CM_DECODE_PGP	used for decoding PGP messages
  *	M_CM_CHARCONV	perform character set conversion 
@@ -495,6 +505,16 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
 	hdr->attach_del = 0;
 	hdr->lines = new_lines;
 	body->offset = new_offset;
+
+	/* update the total size of the mailbox to reflect this deletion */
+	Context->size -= body->length - new_length;
+	/*
+	 * if the message is visible, update the visible size of the mailbox
+	 * as well.
+	 */
+	if (Context->v2r[hdr->msgno] != -1)
+	  Context->vsize -= body->length - new_length;
+
 	body->length = new_length;
 	mutt_free_body (&body->parts);
       }
@@ -519,28 +539,43 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
       s.prefix = prefix;
     if (flags & M_CM_DISPLAY)
       s.flags |= M_DISPLAY;
+    if (flags & M_CM_PRINTING)
+      s.flags |= M_PRINTING;
     if (flags & M_CM_WEED)
       s.flags |= M_WEED;
     if (flags & M_CM_CHARCONV)
       s.flags |= M_CHARCONV;
     
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     if (flags & M_CM_VERIFY)
       s.flags |= M_VERIFY;
 #endif
 
     mutt_body_handler (body, &s);
   }
-#ifdef HAVE_PGP
-  else if ((flags & M_CM_DECODE_PGP) && (hdr->pgp & PGPENCRYPT) &&
-      hdr->content->type == TYPEMULTIPART)
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  else if ((flags & M_CM_DECODE_CRYPT) && (hdr->security & ENCRYPT))
   {
     BODY *cur;
     FILE *fp;
 
-    if (pgp_decrypt_mime (fpin, &fp, hdr->content, &cur))
-      return (-1);
-    fputs ("Mime-Version: 1.0\n", fpout);
+#ifdef HAVE_PGP
+    if ((flags & M_CM_DECODE_PGP) && (hdr->security & APPLICATION_PGP) &&
+	hdr->content->type == TYPEMULTIPART)
+    {
+      if (pgp_decrypt_mime (fpin, &fp, hdr->content, &cur))
+	return (-1);
+      fputs ("Mime-Version: 1.0\n", fpout);
+    }
+#endif
+#ifdef HAVE_SMIME
+    if ((flags & M_CM_DECODE_SMIME) && (hdr->security & APPLICATION_SMIME)
+	     && hdr->content->type == TYPEAPPLICATION)
+    {
+      if (smime_decrypt_mime (fpin, &fp, hdr->content, &cur))
+	return (-1);
+    }
+#endif
     mutt_write_mime_header (cur, fpout);
     fputc ('\n', fpout);
 
