@@ -39,6 +39,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 
 
 
@@ -450,6 +454,13 @@ int mutt_index_menu (void)
      */
     if (option (OPTNEEDRESORT) && Context && Context->msgcount)
       resort_index (menu);
+    
+    if (option (OPTREDRAWTREE) && Context && Context->msgcount && (Sort & SORT_MASK) == SORT_THREADS)
+    {
+      mutt_draw_tree (Context);
+      menu->redraw |= REDRAW_STATUS;
+      unset_option (OPTREDRAWTREE);
+    }
 
     if (Context && !attach_msg)
     {
@@ -581,6 +592,11 @@ int mutt_index_menu (void)
 	menu->menu = MENU_MAIN;
 	SigWinch = 0;
 	menu->top = 0; /* so we scroll the right amount */
+	/*
+	 * force a real complete redraw.  clrtobot() doesn't seem to be able
+	 * to handle every case without this.
+	 */
+	clearok(stdscr,TRUE);
 	continue;
       }
 #endif
@@ -800,7 +816,7 @@ int mutt_index_menu (void)
 	    menu->current = 0;
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	  if ((Sort & SORT_MASK) == SORT_THREADS)
-	    mutt_linearize_tree (Context, 0);
+	    mutt_draw_tree (Context);
 	}
 	break;	  
 
@@ -1207,25 +1223,25 @@ int mutt_index_menu (void)
       case OP_SAVE:
       case OP_DECODE_COPY:
       case OP_DECODE_SAVE:
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_DECRYPT_COPY:
       case OP_DECRYPT_SAVE:
 #endif
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
         if (mutt_save_message (tag ? NULL : CURHDR,
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (op == OP_DECRYPT_SAVE) ||
 #endif
 			       (op == OP_SAVE) || (op == OP_DECODE_SAVE),
 			       (op == OP_DECODE_SAVE) || (op == OP_DECODE_COPY),
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (op == OP_DECRYPT_SAVE) || (op == OP_DECRYPT_COPY) ||
 #endif
 			       0,
 			       &menu->redraw) == 0 &&
 	    (op == OP_SAVE || op == OP_DECODE_SAVE
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 	     || op == OP_DECRYPT_SAVE
 #endif
 	     ))
@@ -1723,10 +1739,10 @@ int mutt_index_menu (void)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_FORGET_PASSPHRASE:
 
-	mutt_forget_passphrase ();
+	crypt_forget_passphrase ();
 	break;
 #endif /* HAVE_PGP */
 
@@ -1770,15 +1786,21 @@ int mutt_index_menu (void)
 	ci_send_message (SENDKEY, NULL, NULL, NULL, NULL);
 	menu->redraw = REDRAW_FULL;
 	break;
+#endif /* HAVE_PGP */
+
       
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_EXTRACT_KEYS:
       
         CHECK_MSGCOUNT;
         CHECK_VISIBLE;
-        pgp_extract_keys_from_messages(tag ? NULL : CURHDR);
+        crypt_extract_keys_from_messages(tag ? NULL : CURHDR);
         menu->redraw = REDRAW_FULL;
         break;
 
+#endif /* HAVE_PGP || HAVE_SMIME */
+
+#ifdef HAVE_PGP
       case OP_CHECK_TRADITIONAL:
       
         CHECK_MSGCOUNT; 
@@ -1952,6 +1974,10 @@ int mutt_index_menu (void)
 	mutt_version ();
 	break;
 
+      case OP_BUFFY_LIST:
+	mutt_buffy_list ();
+	break;
+
       case OP_VIEW_ATTACHMENTS:
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
