@@ -37,6 +37,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 
 #define BUFI_SIZE 1000
 #define BUFO_SIZE 2000
@@ -76,10 +80,10 @@ static void state_prefix_put (const char *d, size_t dlen, STATE *s)
     fwrite (d, dlen, 1, s->fpout);
 }
 
-static void convert_to_state(iconv_t cd, char *bufi, size_t *l, STATE *s)
+void mutt_convert_to_state(iconv_t cd, char *bufi, size_t *l, STATE *s)
 {
   char bufo[BUFO_SIZE];
-  const char *ib;
+  ICONV_CONST char *ib;
   char *ob;
   size_t ibl, obl;
 
@@ -140,11 +144,11 @@ void mutt_decode_xbit (STATE *s, long len, int istext, iconv_t cd)
 
       bufi[l++] = c;
       if (l == sizeof (bufi))
-	convert_to_state (cd, bufi, &l, s);
+	mutt_convert_to_state (cd, bufi, &l, s);
     }
 
-    convert_to_state (cd, bufi, &l, s);
-    convert_to_state (cd, 0, 0, s);
+    mutt_convert_to_state (cd, bufi, &l, s);
+    mutt_convert_to_state (cd, 0, 0, s);
 
     state_reset_prefix (s);
   }
@@ -169,7 +173,8 @@ static int qp_decode_triple (char *s, char *d)
   return -1;
 }
 
-static void qp_decode_line (char *dest, char *src, size_t *l)
+static void qp_decode_line (char *dest, char *src, size_t *l,
+			    int last)
 {
   char *d, *s;
   char c;
@@ -177,16 +182,6 @@ static void qp_decode_line (char *dest, char *src, size_t *l)
   int kind;
   int soft = 0;
 
-  /* chop trailing whitespace */
-
-  if (*src)
-  {
-    s = src + strlen(src) - 1;
-    while (s >= src && ISSPACE(*s))
-      s--;
-    *(++s) = '\0';
-  }
-
   /* decode the line */
   
   for (d = dest, s = src; *s;)
@@ -199,7 +194,7 @@ static void qp_decode_line (char *dest, char *src, size_t *l)
     }
   }
 
-  if (!soft)
+  if (!soft && last == '\n')
     *d++ = '\n';
   
   *d = '\0';
@@ -221,7 +216,7 @@ static void qp_decode_line (char *dest, char *src, size_t *l)
  * result of qp_decode_line.
  * 
  * Finally, at soft line breaks, some part of a multibyte character
- * may have been left over by convert_to_state().  This shouldn't
+ * may have been left over by mutt_convert_to_state().  This shouldn't
  * be more than 6 characters, so STRING + 7 should be sufficient
  * memory to store the decoded data.
  * 
@@ -235,39 +230,52 @@ void mutt_decode_quoted (STATE *s, long len, int istext, iconv_t cd)
   char line[STRING];
   char decline[2*STRING];
   size_t l = 0;
-  size_t l2;
+  size_t linelen;      /* number of input bytes in `line' */
   size_t l3;
-
+  
+  int last;    /* store the last character in the input line */
+  
   if (istext)
     state_set_prefix(s);
 
   while (len > 0)
   {
-    if (fgets (line, sizeof (line), s->fpin) == NULL)
+    last = 0;
+    
+    /*
+     * It's ok to use a fixed size buffer for input, even if the line turns
+     * out to be longer than this.  Just process the line in chunks.  This
+     * really shouldn't happen according the MIME spec, since Q-P encoded
+     * lines are at most 76 characters, but we should be liberal about what
+     * we accept.
+     */
+    if (fgets (line, MIN ((ssize_t)sizeof (line), len + 1), s->fpin) == NULL)
       break;
 
-    len -= (l2 = strlen (line));
+    linelen = strlen(line);
+    len -= linelen;
 
-    /* 
-     * Skip over pending input data until end of line - at this
-     * point, the only thing we may see is trailing whitespace,
-     * i.e. garbage.
+    /*
+     * inspect the last character we read so we can tell if we got the
+     * entire line.
      */
+    last = linelen ? line[linelen - 1] : 0;
 
-    if (l2 && line[l2 - 1] != '\n')
-      while (len > 0 && fgetc (s->fpin) != '\n')
-	len--;
+    /* chop trailing whitespace if we got the full line */
+    if (last == '\n')
+    {
+      while (linelen > 0 && ISSPACE (line[linelen-1]))
+       linelen--;
+      line[linelen]=0;
+    }
 
-    /* 
-     * decode and do character set conversion
-     */
-    
-    qp_decode_line (decline + l, line, &l3);
+    /* decode and do character set conversion */
+    qp_decode_line (decline + l, line, &l3, last);
     l += l3;
-    convert_to_state (cd, decline, &l, s);
+    mutt_convert_to_state (cd, decline, &l, s);
   }
 
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, 0, 0, s);
   state_reset_prefix(s);
 }
 
@@ -342,13 +350,13 @@ void mutt_decode_base64 (STATE *s, long len, int istext, iconv_t cd)
       bufi[l++] = ch;
     
     if (l + 8 >= sizeof (bufi))
-      convert_to_state (cd, bufi, &l, s);
+      mutt_convert_to_state (cd, bufi, &l, s);
   }
 
   if (cr) bufi[l++] = '\r';
 
-  convert_to_state (cd, bufi, &l, s);
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, bufi, &l, s);
+  mutt_convert_to_state (cd, 0, 0, s);
 
   state_reset_prefix(s);
 }
@@ -401,13 +409,13 @@ void mutt_decode_uuencoded (STATE *s, long len, int istext, iconv_t cd)
 	if (c == linelen)
 	  break;
       }
-      convert_to_state (cd, bufi, &k, s);
+      mutt_convert_to_state (cd, bufi, &k, s);
       pt++;
     }
   }
 
-  convert_to_state (cd, bufi, &k, s);
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, bufi, &k, s);
+  mutt_convert_to_state (cd, 0, 0, s);
   
   state_reset_prefix(s);
 }
@@ -931,7 +939,8 @@ static void flowed_stuff (STATE *s, char *cont, int level)
     if (*cont && !level && !mutt_strcmp (Pager, "builtin") && flowed_maybe_quoted (cont))
       state_puts ("\033[0m",s);
   }
-  else if ((*cont == ' ') || (*cont == '>') || (!level && !mutt_strncmp (cont, "From ", 5)))
+  else if ((!(s->flags & M_PRINTING)) && 
+	   ((*cont == ' ') || (*cont == '>') || (!level && !mutt_strncmp (cont, "From ", 5))))
     state_putc (' ', s);
 }
 
@@ -1380,7 +1389,7 @@ int mutt_can_decode (BODY *a)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
     if (ascii_strcasecmp (a->subtype, "signed") == 0 ||
 	ascii_strcasecmp (a->subtype, "encrypted") == 0)
       return (1);
@@ -1402,16 +1411,24 @@ int mutt_can_decode (BODY *a)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
   else if (a->type == TYPEAPPLICATION)
   {
+#ifdef HAVE_PGP
     if (mutt_is_application_pgp(a))
       return (1);
+#ifdef HAVE_SMIME
+    if (mutt_is_application_smime(a))
+      return (1);
+#endif
+#endif
   }
 #endif
 
 
 
+
+
   return (0);
 }
 
@@ -1703,7 +1720,7 @@ static void external_body_handler (BODY *b, STATE *s)
 
 void mutt_decode_attachment (BODY *b, STATE *s)
 {
-  int istext = mutt_is_text_type (b->type, b->subtype);
+  int istext = mutt_is_text_part (b);
   iconv_t cd = (iconv_t)(-1);
 
   if (istext && s->flags & M_CHARCONV)
@@ -1768,6 +1785,11 @@ void mutt_body_handler (BODY *b, STATE *s)
       /* avoid copying this part twice since removing the transfer-encoding is
        * the only operation needed.
        */
+#ifdef HAVE_PGP
+      if (mutt_is_application_pgp (b))
+	handler = pgp_application_pgp_handler;
+      else
+#endif	
       if (ascii_strcasecmp ("flowed", mutt_get_parameter ("format", b->parameter)) == 0)
 	handler = text_plain_flowed_handler;
       else
@@ -1792,9 +1814,9 @@ void mutt_body_handler (BODY *b, STATE *s)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     char *p;
-#endif /* HAVE_PGP */
+#endif /* HAVE_(PGP||SMIME) */
 
 
 
@@ -1803,21 +1825,18 @@ void mutt_body_handler (BODY *b, STATE *s)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     else if (ascii_strcasecmp ("signed", b->subtype) == 0)
     {
       p = mutt_get_parameter ("protocol", b->parameter);
 
       if (!p)
         mutt_error _("Error: multipart/signed has no protocol.");
-      else if (ascii_strcasecmp ("application/pgp-signature", p) == 0 ||
-	       ascii_strcasecmp ("multipart/mixed", p) == 0)
-      {
-	if (s->flags & M_VERIFY)
-	  handler = pgp_signed_handler;
-      }
+      else if (s->flags & M_VERIFY)
+	handler = mutt_signed_handler;
     }
-    else if (ascii_strcasecmp ("encrypted", b->subtype) == 0)
+#ifdef HAVE_PGP
+    else if (mutt_strcasecmp ("encrypted", b->subtype) == 0)
     {
       p = mutt_get_parameter ("protocol", b->parameter);
 
@@ -1827,7 +1846,7 @@ void mutt_body_handler (BODY *b, STATE *s)
         handler = pgp_encrypted_handler;
     }
 #endif /* HAVE_PGP */
-
+#endif /* HAVE_(PGP||SMIME) */
 
 
     if (!handler)
@@ -1836,13 +1855,20 @@ void mutt_body_handler (BODY *b, STATE *s)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
   else if (b->type == TYPEAPPLICATION)
   {
-    if (mutt_is_application_pgp(b))
+#ifdef HAVE_PGP
+    if (mutt_is_application_pgp (b))
       handler = pgp_application_pgp_handler;
-  }
 #endif /* HAVE_PGP */
+#ifdef HAVE_SMIME
+    if (mutt_is_application_smime(b))
+      handler = smime_application_smime_handler;
+#endif /* HAVE_SMIME */
+  }
+#endif /* HAVE_(PGP||SMIME) */
+
 
 
 
@@ -1853,7 +1879,7 @@ void mutt_body_handler (BODY *b, STATE *s)
     /* see if we need to decode this part before processing it */
     if (b->encoding == ENCBASE64 || b->encoding == ENCQUOTEDPRINTABLE ||
 	b->encoding == ENCUUENCODED || plaintext || 
-	mutt_is_text_type (b->type, b->subtype))	/* text subtypes may
+	mutt_is_text_part (b))				/* text subtypes may
 							 * require character
 							 * set conversion even
 							 * with 8bit encoding.
