@@ -86,11 +86,14 @@ static void mutt_usage (void)
 
   puts _(
 "usage: mutt [ -nRyzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
+       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -Q <query> [ -Q <query> ] [...]\n\
+       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -A <alias> [ -A <alias> ] [...]\n\
        mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
        mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
        mutt -v[v]\n\
 \n\
 options:\n\
+  -A <alias>\texpand the given alias\n\
   -a <file>\tattach a file to the message\n\
   -b <address>\tspecify a blind carbon-copy (BCC) address\n\
   -c <address>\tspecify a carbon-copy (CC) address\n\
@@ -102,6 +105,7 @@ options:\n\
   -m <type>\tspecify a default mailbox type\n\
   -n\t\tcauses Mutt not to read the system Muttrc\n\
   -p\t\trecall a postponed message\n\
+  -Q <variable>\tquery a configuration variable\n\
   -R\t\topen mailbox in read-only mode\n\
   -s <subj>\tspecify a subject (must be in quotes if it has spaces)\n\
   -v\t\tshow version and compile-time definitions\n\
@@ -139,6 +143,11 @@ static void show_version (void)
   printf (" [using slang %d]", SLANG_VERSION);
 #endif
 
+#ifdef _LIBICONV_VERSION
+  printf (" [using libiconv %d.%d]", _LIBICONV_VERSION >> 8,
+	  _LIBICONV_VERSION & 0xff);
+#endif
+  
   puts (_("\nCompile options:"));
 
 #ifdef DOMAIN
@@ -225,6 +234,11 @@ static void show_version (void)
 	"+USE_SASL  "
 #else
 	"-USE_SASL  "
+#endif
+#ifdef USE_SASL2
+	"+USE_SASL2  "
+#else
+	"-USE_SASL2  "
 #endif
 	"\n"
 	
@@ -298,6 +312,12 @@ static void show_version (void)
 	"-HAVE_PGP  "
 #endif
 
+#ifdef HAVE_SMIME
+	"+HAVE_SMIME  "
+#else
+	"-HAVE_SMIME  "
+#endif
+
 #ifdef BUFFY_SIZE
 	"+BUFFY_SIZE "
 #else
@@ -449,6 +469,8 @@ int main (int argc, char **argv)
   HEADER *msg = NULL;
   LIST *attach = NULL;
   LIST *commands = NULL;
+  LIST *queries = NULL;
+  LIST *alias_queries = NULL;
   int sendflags = 0;
   int flags = 0;
   int version = 0;
@@ -483,9 +505,12 @@ int main (int argc, char **argv)
   memset (Options, 0, sizeof (Options));
   memset (QuadOptions, 0, sizeof (QuadOptions));
   
-  while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
+  while ((i = getopt (argc, argv, "A:a:b:F:f:c:d:e:H:s:i:hm:npQ:RvxyzZ")) != EOF)
     switch (i)
     {
+      case 'A':
+        alias_queries = mutt_add_list (alias_queries, optarg);
+        break;
       case 'a':
 	attach = mutt_add_list (attach, optarg);
 	break;
@@ -545,6 +570,10 @@ int main (int argc, char **argv)
 	sendflags |= SENDPOSTPONED;
 	break;
 
+      case 'Q':
+        queries = mutt_add_list (queries, optarg);
+        break;
+      
       case 'R':
 	flags |= M_RO; /* read-only mode */
 	break;
@@ -592,7 +621,7 @@ int main (int argc, char **argv)
   }
 
   /* Check for a batch send. */
-  if (!isatty (0))
+  if (!isatty (0) || queries || alias_queries)
   {
     set_option (OPTNOCURSES);
     sendflags = SENDBATCH;
@@ -607,6 +636,26 @@ int main (int argc, char **argv)
   mutt_init (flags & M_NOSYSRC, commands);
   mutt_free_list (&commands);
 
+  if (queries)
+    return mutt_query_variables (queries);
+
+  if (alias_queries)
+  {
+    int rv = 0;
+    ADDRESS *a;
+    for (; alias_queries; alias_queries = alias_queries->next)
+    {
+      if ((a = mutt_lookup_alias (alias_queries->data)))
+	mutt_write_address_list (a, stdout, 0);
+      else
+      {
+	rv = 1;
+	printf ("%s\n", alias_queries->data);
+      }
+    }
+    return rv;
+  }
+  
   if (newMagic)
     mx_set_magic (newMagic);
 
@@ -817,19 +866,15 @@ int main (int argc, char **argv)
 
     if (flags & M_IGNORE)
     {
-      struct stat st;
-
       /* check to see if there are any messages in the folder */
-      if (stat (folder, &st) != 0)
+      switch (mx_check_empty (folder))
       {
-	mutt_endwin (strerror (errno));
-	exit (1);
-      }
-
-      if (st.st_size == 0)
-      {
-	mutt_endwin _("Mailbox is empty.");
-	exit (1);
+	case -1:
+	  mutt_endwin (strerror (errno));
+	  exit (1);
+	case 1:
+	  mutt_endwin _("Mailbox is empty.");
+	  exit (1);
       }
     }
 
