@@ -92,7 +92,7 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	    if (option (OPTASCIICHARS))
 	      addch ('-');
 	    else if (Charset_is_utf8)
-	      addstr ("\342\224\200"); /* FIXME: this is WACS_HLINE but should be WACS_TTEE */
+	      addstr ("\342\224\254"); /* WACS_TTEE */
 	    else
 	      addch (ACS_TTEE);
 	    break;
@@ -100,7 +100,7 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	    if (option (OPTASCIICHARS))
 	      addch ('-');
 	    else if (Charset_is_utf8)
-	      addstr ("\342\224\200"); /* FIXME: this is WACS_HLINE but should be WACS_BTEE */
+	      addstr ("\342\224\264"); /* WACS_BTEE */
 	    else
 	      addch (ACS_BTEE);
 	    break;
@@ -148,30 +148,13 @@ static void menu_make_entry (char *s, int l, MUTTMENU *menu, int i)
     menu->make_entry (s, l, menu, i);
 }
 
-void menu_pad_string (char *s, size_t l)
+void menu_pad_string (char *s, size_t n)
 {
-  size_t n = mutt_strlen (s);
   int shift = option (OPTARROWCURSOR) ? 3 : 0;
-  
-  l--; /* save room for the terminal \0 */
-  if (l > COLS - shift)
-    l = COLS - shift;
+  int cols = COLS - shift;
 
-  /* Let's just pad the string anyway ... */
-  mutt_format_string (s, INT_MAX, l, l, 0, ' ', s, n, 1);
-  return;
-
-#if !defined (HAVE_BKGDSET) && !defined (USE_SLANG_CURSES)
-  /* we have to pad the string with blanks to the end of line */
-  if (n < l)
-  {
-    while (n < l)
-      s[n++] = ' ';
-    s[n] = 0;
-  }
-  else
-#endif
-    s[l] = 0;
+  mutt_format_string (s, n, cols, cols, 0, ' ', s, strlen (s), 1);
+  s[n - 1] = 0;
 }
 
 void menu_redraw_full (MUTTMENU *menu)
@@ -232,7 +215,8 @@ void menu_redraw_index (MUTTMENU *menu)
 
 	if (i == menu->current)
 	{
-	  SETCOLOR (MT_COLOR_INDICATOR);
+          attrset (menu->color (i));
+	  ADDCOLOR (MT_COLOR_INDICATOR);
 	  addstr ("->");
           attrset (menu->color (i));
 	  addch (' ');
@@ -245,14 +229,14 @@ void menu_redraw_index (MUTTMENU *menu)
       }
       else
       {
+        attrset (menu->color (i));
+            
 	if (i == menu->current)
 	{
-	  SETCOLOR (MT_COLOR_INDICATOR);
+	  ADDCOLOR (MT_COLOR_INDICATOR);
 	  BKGDSET (MT_COLOR_INDICATOR);
 	}
-        else
-          attrset (menu->color (i));
-            
+
 	CLEARLINE (i - menu->top + menu->offset);
 	print_enriched_string (menu->color(i), (unsigned char *) buf, i != menu->current);
         SETCOLOR (MT_COLOR_NORMAL);
@@ -297,7 +281,8 @@ void menu_redraw_motion (MUTTMENU *menu)
 
     /* now draw it in the new location */
     move (menu->current + menu->offset - menu->top, 0);
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
     SETCOLOR (MT_COLOR_NORMAL);
   }
@@ -313,7 +298,8 @@ void menu_redraw_motion (MUTTMENU *menu)
     /* now draw the new one to reflect the change */
     menu_make_entry (buf, sizeof (buf), menu, menu->current);
     menu_pad_string (buf, sizeof (buf));
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     CLEARLINE (menu->current - menu->top + menu->offset);
     print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
@@ -336,7 +322,8 @@ void menu_redraw_current (MUTTMENU *menu)
     int attr = menu->color (menu->current);
     attrset (attr);
     clrtoeol ();
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
     attrset (attr);
     addch (' ');
@@ -346,7 +333,8 @@ void menu_redraw_current (MUTTMENU *menu)
   }
   else
   {
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     clrtoeol ();
     print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
@@ -895,6 +883,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       mutt_resize_screen ();
       menu->redraw = REDRAW_FULL;
       SigWinch = 0;
+      clearok(stdscr,TRUE);/*force complete redraw*/
     }
 #endif
 
