@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -50,6 +50,13 @@
 #include "hash.h"
 #include "charset.h"
 
+#ifndef HAVE_WC_FUNCS
+# ifdef MB_LEN_MAX
+#  undef MB_LEN_MAX
+# endif
+# define MB_LEN_MAX 16
+#endif
+
 #ifdef SUBVERSION
 # define MUTT_VERSION (VERSION SUBVERSION)
 #else  
@@ -81,6 +88,7 @@
 #define CH_NOLEN	(1<<12) /* don't write Content-Length: and Lines: */
 #define CH_WEED_DELIVERED (1<<13) /* weed eventual Delivered-To headers */
 #define CH_FORCE_FROM	(1<<14)	/* give CH_FROM precedence over CH_WEED? */
+#define CH_NOQFROM	(1<<15)	/* give CH_FROM precedence over CH_WEED? */
 
 /* flags for mutt_enter_string() */
 #define  M_ALIAS   1      /* do alias "completion" by calling up the alias-menu */
@@ -139,10 +147,11 @@ typedef enum
 #define M_CHARSETHOOK	(1<<5)
 #define M_ICONVHOOK 	(1<<6)
 #define M_MESSAGEHOOK	(1<<7)
-#ifdef HAVE_PGP
-#define M_PGPHOOK	(1<<8)
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+#define M_CRYPTHOOK	(1<<8)
 #endif
 #define M_ACCOUNTHOOK	(1<<9)
+#define M_REPLYHOOK	(1<<10)
 
 /* tree characters for linearize_tree and print_enriched_string */
 #define M_TREE_LLCORNER		1
@@ -216,9 +225,12 @@ enum
   M_PERSONAL_RECIP,
   M_PERSONAL_FROM,
   M_ADDRESS,
+#if defined (HAVE_PGP) || defined (HAVE_SMIME)
+  M_CRYPT_SIGN,
+  M_CRYPT_VERIFIED,
+  M_CRYPT_ENCRYPT,
+#endif
 #ifdef HAVE_PGP
-  M_PGP_SIGN,
-  M_PGP_ENCRYPT,
   M_PGP_KEY,
 #endif
   M_XLABEL,
@@ -253,10 +265,12 @@ enum
 enum
 {
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP)||  defined(HAVE_SMIME)
   OPT_VERIFYSIG,      /* verify PGP signatures */
+#ifdef HAVE_PGP
   OPT_PGPTRADITIONAL, /* create old-style PGP messages */
 #endif
+#endif
 
 #ifdef USE_SSL
   OPT_SSLSTARTTLS,
@@ -280,6 +294,7 @@ enum
   OPT_RECALL,
   OPT_SUBJECT,
   OPT_MIMEFWDREST,
+  OPT_FORWEDIT,
   OPT_MAX
 };
 
@@ -336,7 +351,10 @@ enum
   OPTHEADER,
   OPTHELP,
   OPTHIDDENHOST,
+  OPTHIDELIMITED,
   OPTHIDEMISSING,
+  OPTHIDETOPLIMITED,
+  OPTHIDETOPMISSING,
   OPTIGNORELISTREPLYTO,
 #ifdef USE_IMAP
   OPTIMAPLSUB,
@@ -364,6 +382,7 @@ enum
   OPTMETOO,
   OPTMHPURGE,
   OPTMIMEFORWDECODE,
+  OPTNARROWTREE,
   OPTPAGERSTOP,
   OPTPIPEDECODE,
   OPTPIPESPLIT,
@@ -411,14 +430,22 @@ enum
 
   /* PGP options */
   
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  OPTCRYPTAUTOSIGN,
+  OPTCRYPTAUTOENCRYPT,
+  OPTCRYPTREPLYENCRYPT,
+  OPTCRYPTREPLYSIGN,
+  OPTCRYPTREPLYSIGNENCRYPTED,
+  OPTCRYPTTIMESTAMP,
+#ifdef HAVE_SMIME
+  OPTSMIMEISDEFAULT,
+  OPTASKCERTLABEL,
+  OPTSDEFAULTDECRYPTKEY,
+#endif
 #ifdef HAVE_PGP
-  OPTPGPAUTOSIGN,
-  OPTPGPAUTOENCRYPT,
   OPTPGPIGNORESUB,
   OPTPGPLONGIDS,
-  OPTPGPREPLYENCRYPT,
-  OPTPGPREPLYSIGN,
-  OPTPGPREPLYSIGNENCRYPTED,
+#endif
 #if 0
   OPTPGPENCRYPTSELF,
 #endif
@@ -453,6 +480,7 @@ enum
 			 * 	    external program.
 			 */
   OPTMENUCALLER,	/* (pseudo) tell menu to give caller a take */
+  OPTREDRAWTREE,	/* (pseudo) redraw the thread tree */
 #ifdef HAVE_PGP
   OPTPGPCHECKTRUST,	/* (pseudo) used by pgp_select_key () */
   OPTDONTHANDLEPGPKEYS,	/* (pseudo) used to extract PGP keys */
@@ -600,8 +628,9 @@ typedef struct body
 				 * set when in send-mode.
 				 */
 
-#ifdef HAVE_PGP
-  unsigned int goodsig : 1;	/* good PGP signature */
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  unsigned int goodsig : 1;	/* good cryptographic signature */
+  unsigned int badsig : 1;	/* bad cryptographic signature (needed to check encrypted s/mime-signatures */
 #endif
 
   unsigned int collapsed : 1;	/* used by recvattach */
@@ -610,8 +639,9 @@ typedef struct body
 
 typedef struct header
 {
-#ifdef HAVE_PGP
-  unsigned int pgp : 4;
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  unsigned int security : 7;  /* bit 0-4: flags, bit 5,6: application.
+				 see: crypt.h pgplib.h, smime.h */
 #endif
 
   unsigned int mime : 1;    		/* has a Mime-Version header? */
@@ -687,6 +717,10 @@ typedef struct thread
   unsigned int duplicate_thread : 1;
   unsigned int sort_children : 1;
   unsigned int check_subject : 1;
+  unsigned int visible : 1;
+  unsigned int deep : 1;
+  unsigned int subtree_visible : 2;
+  unsigned int next_subtree_visible : 1;
   struct thread *parent;
   struct thread *child;
   struct thread *next;
@@ -791,7 +825,7 @@ typedef struct
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 #define M_VERIFY	(1<<1) /* perform signature verification */
 #endif
 
