@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 #ifdef USE_IMAP
 #include "imap.h"
 #endif
@@ -485,7 +489,7 @@ int mx_access (const char* path, int flags)
   return access (path, flags);
 }
 
-static int mx_open_mailbox_append (CONTEXT *ctx)
+static int mx_open_mailbox_append (CONTEXT *ctx, int flags)
 {
   struct stat sb;
 
@@ -581,7 +585,7 @@ static int mx_open_mailbox_append (CONTEXT *ctx)
   {
     case M_MBOX:
     case M_MMDF:
-      if ((ctx->fp = fopen (ctx->path, "a")) == NULL ||
+    if ((ctx->fp = safe_fopen (ctx->path, flags & M_NEWFOLDER ? "w" : "a")) == NULL ||
 	  mbox_lock_mailbox (ctx, 1, 1) != 0)
       {
 	if (!ctx->fp)
@@ -636,9 +640,9 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
   if (flags & M_READONLY)
     ctx->readonly = 1;
 
-  if (flags & M_APPEND)
+  if (flags & (M_APPEND|M_NEWFOLDER))
   {
-    if (mx_open_mailbox_append (ctx) != 0)
+    if (mx_open_mailbox_append (ctx, flags) != 0)
     {
       mx_fastclose_mailbox (ctx);
       if (!pctx)
@@ -953,8 +957,7 @@ int mx_close_mailbox (CONTEXT *ctx, int *index_hint)
         {
 	  if (mutt_append_message (&f, ctx, ctx->hdrs[i], 0, CH_UPDATE_LEN) == 0)
 	  {
-	    ctx->hdrs[i]->deleted = 1;
-	    ctx->deleted++;
+	    mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, 1);
 	  }
 	  else
 	  {
@@ -1251,7 +1254,6 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
   MESSAGE *msg;
   int (*func) (MESSAGE *, CONTEXT *, HEADER *);
   ADDRESS *p = NULL;
-  time_t t;
 
   switch (dest->magic)
   {
@@ -1285,7 +1287,11 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     msg->flags.flagged = hdr->flagged;
     msg->flags.replied = hdr->replied;
     msg->flags.read    = hdr->read;
+    msg->received = hdr->received;
   }
+
+  if(msg->received == 0)
+    time(&msg->received);
   
   if (func (msg, dest, hdr) == 0)
   {
@@ -1303,15 +1309,9 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
 	  p = hdr->env->sender;
 	else
 	  p = hdr->env->from;
-
-	if (!hdr->received)
-	  hdr->received = time (NULL);
-	t = hdr->received;
       }
-      else
-	t = time (NULL);
 
-      fprintf (msg->fp, "From %s %s", p ? p->mailbox : NONULL(Username), ctime (&t));
+      fprintf (msg->fp, "From %s %s", p ? p->mailbox : NONULL(Username), ctime (&msg->received));
     }
   }
   else
@@ -1356,8 +1356,9 @@ int mx_check_mailbox (CONTEXT *ctx, int *index_hint, int lock)
 
 
       case M_MH:
-      case M_MAILDIR:
 	return (mh_check_mailbox (ctx, index_hint));
+      case M_MAILDIR:
+	return (maildir_check_mailbox (ctx, index_hint));
 
 #ifdef USE_IMAP
       case M_IMAP:
@@ -1570,10 +1571,10 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     /* NOTE: this _must_ be done before the check for mailcap! */
-    h->pgp = pgp_query (h->content);
-#endif /* HAVE_PGP */
+    h->security = crypt_query (h->content);
+#endif /* HAVE_PGP || HAVE_SMIME */
 
     if (!ctx->pattern)
     {
@@ -1625,3 +1626,28 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
     }
   }
 }
+
+/*
+ * Return:
+ * 1 if the specified mailbox contains 0 messages.
+ * 0 if the mailbox contains messages
+ * -1 on error
+ */
+int mx_check_empty (const char *path)
+{
+  switch (mx_get_magic (path))
+  {
+    case M_MBOX:
+    case M_MMDF:
+    case M_KENDRA:
+      return mbox_check_empty (path);
+    case M_MH:
+      return mh_check_empty (path);
+    case M_MAILDIR:
+      return maildir_check_empty (path);
+    default:
+      errno = EINVAL;
+      return -1;
+  }
+  /* not reached */
+}
