@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 
 
 
@@ -1051,9 +1055,14 @@ static int format_line (struct line_t **lineInfo, int n, unsigned char *buf,
 	   check_attachment_marker ((char *) buf+ch) == 0)
     {
       while (buf[ch++] != '\a')
-	;
+	if (ch >= cnt)
+	  break;
     }
 
+    /* is anything left to do? */
+    if (ch >= cnt)
+      break;
+    
     k = mbrtowc (&wc, (char *)buf+ch, cnt-ch, &mbstate);
     if (k == -2 || k == -1)
     {
@@ -1361,26 +1370,6 @@ display_line (FILE *f, long *last_pos, struct line_t **lineInfo, int n,
   if (!(flags & M_SHOW))
     return 0;
 
-  if (flags & M_SHOWCOLOR)
-  {
-    m = ((*lineInfo)[n].continuation) ? ((*lineInfo)[n].syntax)[0].first : n;
-    if ((*lineInfo)[m].type == MT_COLOR_HEADER)
-      def_color = ((*lineInfo)[m].syntax)[0].color;
-    else
-      def_color = (*lineInfo)[m].type;
-
-    attrset (def_color);
-#ifdef HAVE_BKGDSET
-    bkgdset (def_color | ' ');
-#endif
-    clrtoeol ();
-    SETCOLOR (MT_COLOR_NORMAL);
-    BKGDSET (MT_COLOR_NORMAL);
-    
-  }
-  else
-    clrtoeol ();
-
   /* display the line */
   format_line (lineInfo, n, buf, flags, &a, cnt, &ch, &vch, &col, &special);
 
@@ -1397,6 +1386,25 @@ display_line (FILE *f, long *last_pos, struct line_t **lineInfo, int n,
   if (special || (col != COLS && (flags & (M_SHOWCOLOR | M_SEARCH))))
     resolve_color (*lineInfo, n, vch, flags, 0, &a);
           
+  /*
+   * Fill the blank space at the end of the line with the prevailing color.
+   * ncurses does an implicit clrtoeol() when you do addch('\n') so we have
+   * to make sure to reset the color *after* that
+   */
+  if (flags & M_SHOWCOLOR)
+  {
+    m = ((*lineInfo)[n].continuation) ? ((*lineInfo)[n].syntax)[0].first : n;
+    if ((*lineInfo)[m].type == MT_COLOR_HEADER)
+      def_color = ((*lineInfo)[m].syntax)[0].color;
+    else
+      def_color = ColorDefs[ (*lineInfo)[m].type ];
+
+    attrset (def_color);
+#ifdef HAVE_BKGDSET
+    bkgdset (def_color | ' ');
+#endif
+  }
+
   /* ncurses always wraps lines when you get to the right side of the
    * screen, but S-Lang seems to only wrap if the next character is *not*
    * a newline (grr!).
@@ -1406,6 +1414,17 @@ display_line (FILE *f, long *last_pos, struct line_t **lineInfo, int n,
 #endif
       addch ('\n');
 
+  /*
+   * reset the color back to normal.  This *must* come after the
+   * addch('\n'), otherwise the color for this line will not be
+   * filled to the right margin.
+   */
+  if (flags & M_SHOWCOLOR)
+  {
+    SETCOLOR(MT_COLOR_NORMAL);
+    BKGDSET(MT_COLOR_NORMAL);
+  }
+
   /* build a return code */
   if (!(flags & M_SHOW))
     flags = 0;
@@ -1788,6 +1807,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
       }
 
       SigWinch = 0;
+      clearok(stdscr,TRUE);/*force complete redraw*/
       continue;
     }
 #endif
@@ -1891,6 +1911,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
       case OP_SEARCH_OPPOSITE:
 	if (SearchCompiled)
 	{
+search_next:
 	  if ((!SearchBack && ch==OP_SEARCH_NEXT) ||
 	      (SearchBack &&ch==OP_SEARCH_OPPOSITE))
 	  {
@@ -1933,17 +1954,39 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 
       case OP_SEARCH:
       case OP_SEARCH_REVERSE:
+	buffer[0] = 0;
+	if (mutt_get_field ((SearchBack ? _("Reverse search: ") :
+			  _("Search: ")), buffer, sizeof (buffer),
+			  M_CLEAR) != 0)
+	  break;
+
+	if (!buffer[0])
+	{
+	  if (SearchCompiled)
+	  {
+	    /* do an implicit search-next */
+	    if (ch == OP_SEARCH)
+	      ch = OP_SEARCH_NEXT;
+	    else
+	      ch = OP_SEARCH_OPPOSITE;
+
+	    goto search_next;
+	  }
+	  /*
+	   * preserve old behavior of doing nothing if there is no last
+	   * used pattern.
+	   */
+	  break;
+	}
+
+	strfcpy (searchbuf, buffer, sizeof (searchbuf));
+
 	/* leave SearchBack alone if ch == OP_SEARCH_NEXT */
 	if (ch == OP_SEARCH)
 	  SearchBack = 0;
 	else if (ch == OP_SEARCH_REVERSE)
 	  SearchBack = 1;
 
-	if (mutt_get_field ((SearchBack ? _("Reverse search: ") :
-			  _("Search: ")), searchbuf, sizeof (searchbuf),
-			  M_CLEAR) != 0 || !searchbuf[0])
-	  break;
-
 	if (SearchCompiled)
 	{
 	  regfree (&SearchRE);
@@ -2378,7 +2421,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	redraw = REDRAW_FULL;
 	break;
 
-#ifdef HAVE_PGP      
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_DECRYPT_SAVE:
 #endif
       case OP_SAVE:
@@ -2391,22 +2434,22 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
       case OP_COPY_MESSAGE:
       case OP_DECODE_SAVE:
       case OP_DECODE_COPY:
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_DECRYPT_COPY:
 #endif
 	CHECK_MODE(IsHeader (extra));
 	if (mutt_save_message (extra->hdr,
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (ch == OP_DECRYPT_SAVE) ||
 #endif			       
 			       (ch == OP_SAVE) || (ch == OP_DECODE_SAVE),
 			       (ch == OP_DECODE_SAVE) || (ch == OP_DECODE_COPY),
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 			       (ch == OP_DECRYPT_SAVE) || (ch == OP_DECRYPT_COPY) ||
 #endif
 			       0,
 			       &redraw) == 0 && (ch == OP_SAVE || ch == OP_DECODE_SAVE
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 						 || ch == OP_DECRYPT_SAVE
 #endif
 						 ))
@@ -2495,6 +2538,10 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	mutt_version ();
 	break;
 
+      case OP_BUFFY_LIST:
+	mutt_buffy_list ();
+	break;
+
       case OP_VIEW_ATTACHMENTS:
         if (flags & M_PAGER_ATTACHMENT)
         {
@@ -2512,23 +2559,26 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 
 
 #ifdef HAVE_PGP
-      case OP_FORGET_PASSPHRASE:
-	mutt_forget_passphrase ();
-	break;
-
       case OP_MAIL_KEY:
 	CHECK_MODE(IsHeader(extra));
         CHECK_ATTACH;
 	ci_send_message (SENDKEY, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
-      
+#endif /* HAVE_PGP || HAVE_SMIME */
+
+
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+      case OP_FORGET_PASSPHRASE:
+	crypt_forget_passphrase ();
+	break;
+
       case OP_EXTRACT_KEYS:
         CHECK_MODE(IsHeader(extra));
-        pgp_extract_keys_from_messages(extra->hdr);
+	crypt_extract_keys_from_messages(extra->hdr);
         redraw = REDRAW_FULL;
         break;
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
 
 
 
