@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -41,6 +41,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 #ifdef MIXMASTER
 #include "remailer.h"
 #endif
@@ -343,12 +347,12 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
   mutt_message_hook (ctx, cur, M_MESSAGEHOOK);
 
 #ifdef HAVE_PGP
-  if ((cur->pgp & PGPENCRYPT) && option (OPTFORWDECODE))
+  if ((cur->security & ENCRYPT) && option (OPTFORWDECODE))
   {
     /* make sure we have the user's passphrase before proceeding... */
-    pgp_valid_passphrase ();
+    crypt_valid_passphrase (cur->security);
   }
-#endif /* HAVE_PGP */
+#endif /* HAVE_SMIME */
 
   mutt_forward_intro (out, cur);
 
@@ -397,15 +401,12 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
   int chflags = CH_DECODE;
 
 #ifdef HAVE_PGP
-  if (cur->pgp)
-  {
-    if (cur->pgp & PGPENCRYPT)
+    if (cur->security & ENCRYPT)
     {
       /* make sure we have the user's passphrase before proceeding... */
-      pgp_valid_passphrase ();
+      crypt_valid_passphrase (cur->security);
     }
-  }
-#endif /* HAVE_PGP */
+#endif /* HAVE_SMIME */
 
   mutt_parse_mime_message (ctx, cur);
   mutt_message_hook (ctx, cur, M_MESSAGEHOOK);
@@ -562,6 +563,8 @@ LIST *mutt_make_references(ENVELOPE *e)
 
 void mutt_fix_reply_recipients (ENVELOPE *env)
 {
+  mutt_expand_aliases_env (env);
+
   if (! option (OPTMETOO))
   {
     /* the order is important here.  do the CC: first so that if the
@@ -699,7 +702,6 @@ envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
       return (-1);
     }
 
-    mutt_fix_reply_recipients (env);
     mutt_make_misc_reply_headers (env, ctx, cur, curenv);
     mutt_make_reference_headers (tag ? NULL : curenv, env, ctx);
   }
@@ -801,12 +803,15 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 
 
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP)
+/*  || defined(HAVE_SMIME) */
   else if (flags & SENDKEY) 
   {
     BODY *tmp;
+#ifdef HAVE_PGP
     if ((tmp = pgp_make_key_attachment (NULL)) == NULL)
       return -1;
+#endif
 
     tmp->next = msg->content;
     msg->content = tmp;
@@ -1021,46 +1026,6 @@ int mutt_resend_message (FILE *fp, CONTEXT *ctx, HEADER *cur)
   return ci_send_message (SENDRESEND, msg, NULL, ctx, cur);
 }
 
-#ifdef HAVE_PGP
-
-static int _set_pgp_flags (HEADER *cur)
-{
-  int flags = 0;
-  
-  if (option (OPTPGPREPLYENCRYPT) && cur && cur->pgp & PGPENCRYPT)
-    flags |= PGPENCRYPT;
-  if (option (OPTPGPREPLYSIGN) && cur && cur->pgp & PGPSIGN)
-    flags |= PGPSIGN;
-  if (option (OPTPGPREPLYSIGNENCRYPTED) && cur && cur->pgp & PGPENCRYPT)
-    flags |= PGPSIGN;
-
-  return flags;
-
-}
-
-static int set_pgp_flags (HEADER *cur, CONTEXT *ctx)
-{
-  int i;
-  int flags = 0;
-  
-  if (cur) 
-    return _set_pgp_flags (cur);
-  
-  if (!ctx)
-    return 0;
-    
-  for (i = 0; i < ctx->vcount; i++)
-  {
-    cur = ctx->hdrs[ctx->v2r[i]];
-    if (cur->tagged)
-      flags |= _set_pgp_flags (cur);
-  }
-
-  return flags;
-}
-
-#endif /* HAVE_PGP */
-
 int
 ci_send_message (int flags,		/* send mode */
 		 HEADER *msg,		/* template to use for new message */
@@ -1073,8 +1038,9 @@ ci_send_message (int flags,		/* send mode */
   FILE *tempfp = NULL;
   BODY *pbody;
   int i, killfrom = 0;
+  int fcc_error = 0;
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
   BODY *save_content = NULL;
   BODY *clear_content = NULL;
   char *pgpkeylist = NULL;
@@ -1103,11 +1069,12 @@ ci_send_message (int flags,		/* send mode */
     signas = safe_strdup(PgpSignAs);
 #endif /* HAVE_PGP */
 
-  if (msg)
-  {
-    mutt_expand_aliases_env (msg->env);
-  }
-  else
+  /* Delay expansion of aliases until absolutely necessary--shouldn't
+   * be necessary unless we are prompting the user or about to execute a
+   * send-hook.
+   */
+
+  if (!msg)
   {
     msg = mutt_new_header ();
 
@@ -1136,12 +1103,11 @@ ci_send_message (int flags,		/* send mode */
     pbody->next = msg->content; /* don't kill command-line attachments */
     msg->content = pbody;
     
-    msg->content->type = TYPETEXT;
-    msg->content->subtype = safe_strdup ("plain");
+    mutt_parse_content_type (ContentType, msg->content);
     msg->content->unlink = 1;
     msg->content->use_disp = 0;
     msg->content->disposition = DISPINLINE;
-    if (option (OPTTEXTFLOWED))
+    if (option (OPTTEXTFLOWED) && msg->content->type == TYPETEXT && !ascii_strcasecmp (msg->content->subtype, "plain"))
       mutt_set_parameter ("format", "flowed", &msg->content->parameter);
     
     if (!tempfile)
@@ -1170,7 +1136,15 @@ ci_send_message (int flags,		/* send mode */
     /* we shouldn't have to worry about freeing `msg->env->from' before
      * setting it here since this code will only execute when doing some
      * sort of reply.  the pointer will only be set when using the -H command
-     * line option */
+     * line option.
+     *
+     * We shouldn't have to worry about alias expansion here since we are
+     * either replying to a real or postponed message, therefore no aliases
+     * should exist since the user has not had the opportunity to add
+     * addresses to the list.  We just have to ensure the postponed messages
+     * have their aliases expanded.
+     */
+
     msg->env->from = set_reverse_name (cur->env);
   }
 
@@ -1185,6 +1159,7 @@ ci_send_message (int flags,		/* send mode */
       process_user_recips (msg->env);
       process_user_header (msg->env);
     }
+    mutt_expand_aliases_env (msg->env);
   }
   else if (! (flags & (SENDPOSTPONED|SENDRESEND)))
   {
@@ -1195,6 +1170,9 @@ ci_send_message (int flags,		/* send mode */
     if (option (OPTHDRS))
       process_user_recips (msg->env);
 
+    /* Expand aliases and remove duplicates/crossrefs */
+    mutt_fix_reply_recipients (msg->env);
+
     if (! (flags & SENDMAILX) &&
 	! (option (OPTAUTOEDIT) && option (OPTEDITHDRS)) &&
 	! ((flags & SENDREPLY) && option (OPTFASTREPLY)))
@@ -1214,10 +1192,31 @@ ci_send_message (int flags,		/* send mode */
       killfrom = 1;
     }
 
+    if ((flags & SENDREPLY) && cur)
+    {
+      /* change setting based upon message we are replying to */
+      mutt_message_hook (NULL, cur, M_REPLYHOOK);
+
+      /*
+       * set the replied flag for the message we are generating so that the
+       * user can use ~Q in a send-hook to know when reply-hook's are also
+       * being used.
+       */
+      msg->replied = 1;
+    }
+
     /* change settings based upon recipients */
     
     mutt_message_hook (NULL, msg, M_SENDHOOK);
 
+    /*
+     * Unset the replied flag from the message we are composing since it is
+     * no longer required.  This is done here because the FCC'd copy of
+     * this message was erroneously get the 'R'eplied flag when stored in
+     * a maildir-style mailbox.
+     */
+    msg->replied = 0;
+
     if (killfrom)
     {
       rfc822_free_address (&msg->env->from);
@@ -1244,20 +1243,56 @@ ci_send_message (int flags,		/* send mode */
      * can take effect.
      */
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP)|| defined(HAVE_SMIME)
     if (! (flags & SENDMAILX))
     {
-      if (option (OPTPGPAUTOSIGN))
-	msg->pgp |= PGPSIGN;
-      if (option (OPTPGPAUTOENCRYPT))
-	msg->pgp |= PGPENCRYPT;
-      
-      msg->pgp |= set_pgp_flags (cur, ctx);
-    }
+      if (option (OPTCRYPTAUTOSIGN))
+	msg->security |= SIGN;
+      if (option (OPTCRYPTAUTOENCRYPT))
+	msg->security |= ENCRYPT;
+      if (option (OPTCRYPTREPLYENCRYPT) && cur && cur->security & ENCRYPT)
+	msg->security |= ENCRYPT;
+      if (option (OPTCRYPTREPLYSIGN) && cur && cur->security & SIGN)
+	msg->security |= SIGN;
+      if (option (OPTCRYPTREPLYSIGNENCRYPTED) && cur && cur->security & ENCRYPT)
+	msg->security |= SIGN;
+    }      
 
+    if (msg->security && cur)
+    {
+#ifdef HAVE_SMIME
+      if (cur->security & APPLICATION_SMIME || option (OPTSMIMEISDEFAULT))
+        msg->security |= APPLICATION_SMIME;
+#endif
+#ifdef HAVE_PGP
+      if (cur->security & APPLICATION_PGP)
+      {
+#ifdef HAVE_SMIME
+        if (option (OPTSMIMEISDEFAULT))
+          msg->security &= ~APPLICATION_SMIME;
+#endif
+        msg->security |= APPLICATION_PGP;
+      }
+#ifdef HAVE_SMIME
+      if (~cur->security && !option (OPTSMIMEISDEFAULT))
+#endif
+	msg->security |= APPLICATION_PGP;
 #endif /* HAVE_PGP */
-      
-
+    }
+    else if (msg->security)
+    {
+#ifdef HAVE_PGP
+      msg->security |= APPLICATION_PGP;
+#ifdef HAVE_SMIME
+      if (option (OPTSMIMEISDEFAULT))
+      {
+        msg->security |= APPLICATION_SMIME;
+        msg->security &= ~APPLICATION_PGP;
+      }
+#endif
+#endif /* HAVE_PGP */
+    }
+#endif /* HAVE_PGP || HAVE_SMIME */
 
 
   }
@@ -1287,9 +1322,22 @@ ci_send_message (int flags,		/* send mode */
     
     mutt_update_encoding (msg->content);
 
-    /* If the this isn't a text message, look for a mailcap edit command */
-    if(! (flags & SENDKEY))
+    /*
+     * Select whether or not the user's editor should be called now.  We
+     * don't want to do this when:
+     * 1) we are sending a key/cert
+     * 2) we are forwarding a message and the user doesn't want to edit it.
+     *    This is controled by the quadoption $forward_edit.  However, if
+     *    both $edit_headers and $autoedit are set, we want to ignore the
+     *    setting of $forward_edit because the user probably needs to add the
+     *    recipients.
+     */
+    if (! (flags & SENDKEY) &&
+	((flags & SENDFORWARD) == 0 ||
+	 (option (OPTEDITHDRS) && option (OPTAUTOEDIT)) ||
+	 query_quadoption (OPT_FORWEDIT, _("Edit forwarded message?")) == M_YES))
     {
+      /* If the this isn't a text message, look for a mailcap edit command */
       if (mutt_needs_mailcap (msg->content))
 	mutt_edit_attachment (msg->content);
       else if (!Editor || mutt_strcmp ("builtin", Editor) == 0)
@@ -1320,7 +1368,7 @@ ci_send_message (int flags,		/* send mode */
   /* specify a default fcc.  if we are in batchmode, only save a copy of
    * the message if the value of $copy is yes or ask-yes */
 
-  if (!fcc[0] && !(flags & (SENDRESEND|SENDPOSTPONED)) && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
+  if (!fcc[0] && !(flags & (SENDPOSTPONED)) && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
   {
     /* set the default FCC */
     if (!msg->env->from)
@@ -1344,6 +1392,7 @@ ci_send_message (int flags,		/* send mode */
   {
 main_loop:
 
+    fcc_error = 0; /* reset value since we may have failed before */
     mutt_pretty_mailbox (fcc);
     i = mutt_compose_menu (msg, fcc, sizeof (fcc), cur);
     if (i == -1)
@@ -1414,14 +1463,14 @@ main_loop:
 
   encode_descriptions (msg->content, 1);
   
-#ifdef HAVE_PGP
-  if (msg->pgp)
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (msg->security)  
   {
     /* save the decrypted attachments */
     clear_content = msg->content;
 
-    if ((pgp_get_keys (msg, &pgpkeylist) == -1) ||
-	(pgp_protect (msg, pgpkeylist) == -1))
+    if ((crypt_get_keys (msg, &pgpkeylist) == -1) ||
+	mutt_protect (msg, pgpkeylist) == -1)
     {
       msg->content = mutt_remove_multipart (msg->content);
       
@@ -1472,26 +1521,26 @@ main_loop:
   if (*fcc && mutt_strcmp ("/dev/null", fcc) != 0)
   {
     BODY *tmpbody = msg->content;
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     BODY *save_sig = NULL;
     BODY *save_parts = NULL;
 #endif /* HAVE_PGP */
 
-#ifdef HAVE_PGP
-    if (msg->pgp && option (OPTFCCCLEAR))
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+    if (msg->security && option (OPTFCCCLEAR))
       msg->content = clear_content;
 #endif
 
     /* check to see if the user wants copies of all attachments */
     if (!option (OPTFCCATTACH) && msg->content->type == TYPEMULTIPART)
     {
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       if (mutt_strcmp (msg->content->subtype, "encrypted") == 0 ||
 	  mutt_strcmp (msg->content->subtype, "signed") == 0)
       {
 	if (clear_content->type == TYPEMULTIPART)
 	{
-	  if (!(msg->pgp & PGPENCRYPT) && (msg->pgp & PGPSIGN))
+	  if(!(msg->security & ENCRYPT) && (msg->security & SIGN))
 	  {
 	    /* save initial signature and attachments */
 	    save_sig = msg->content->parts->next;
@@ -1501,7 +1550,7 @@ main_loop:
 	  /* this means writing only the main part */
 	  msg->content = clear_content->parts;
 
-	  if (pgp_protect (msg, pgpkeylist) == -1)
+	  if (mutt_protect (msg, pgpkeylist) == -1)
 	  {
 	    /* we can't do much about it at this point, so
 	     * fallback to saving the whole thing to fcc
@@ -1515,19 +1564,32 @@ main_loop:
 	}
       }
       else
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
 	msg->content = msg->content->parts;
     }
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
 full_fcc:
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
     if (msg->content)
-      mutt_write_fcc (fcc, msg, NULL, 0, NULL);
+    {
+      /* update received time so that when storing to a mbox-style folder
+       * the From_ line contains the current time instead of when the
+       * message was first postponed.
+       */
+      msg->received = time (NULL);
+      if (mutt_write_fcc (fcc, msg, NULL, 0, NULL) == -1)
+      {
+	/*
+	 * Error writing FCC, we should abort sending.
+	 */
+	fcc_error = 1;
+      }
+    }
 
     msg->content = tmpbody;
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
     if (save_sig)
     {
       /* cleanup the second signature structures */
@@ -1548,22 +1610,27 @@ full_fcc:
       mutt_free_body (&save_content);
     }
       
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
   }
 
 
-  if ((i = send_message (msg)) == -1)
+  /*
+   * Don't attempt to send the message if the FCC failed.  Just pretend
+   * the send failed as well so we give the user a chance to fix the
+   * error.
+   */
+  if (fcc_error || (i = send_message (msg)) == -1)
   {
     if (!(flags & SENDBATCH))
     {
-#ifdef HAVE_PGP
-      if ((msg->pgp & PGPENCRYPT) || 
-	  ((msg->pgp & PGPSIGN) && msg->content->type == TYPEAPPLICATION))
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+      if ((msg->security & ENCRYPT) || 
+	  ((msg->security & SIGN)  && msg->content->type == TYPEAPPLICATION))
       {
 	mutt_free_body (&msg->content); /* destroy PGP data */
 	msg->content = clear_content;	/* restore clear text. */
       }
-      else if ((msg->pgp & PGPSIGN) && msg->content->type == TYPEMULTIPART)
+      else if ((msg->security & SIGN) && msg->content->type == TYPEMULTIPART)
       {
 	mutt_free_body (&msg->content->parts->next);		/* destroy sig */
 	msg->content = mutt_remove_multipart (msg->content);	/* remove multipart */
@@ -1583,14 +1650,14 @@ full_fcc:
   else if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
     mutt_message (i == 0 ? _("Mail sent.") : _("Sending in background."));
 
-#ifdef HAVE_PGP
-  if (msg->pgp & PGPENCRYPT)
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (msg->security & ENCRYPT)
   {
     /* cleanup structures from the first encryption */
     mutt_free_body (&clear_content);
     FREE (&pgpkeylist);
   }
-#endif /* HAVE_PGP */
+#endif
 
   if (flags & SENDREPLY)
   {
@@ -1629,3 +1696,5 @@ cleanup:
   return rv;
 
 }
+
+/* vim: set sw=2: */
