@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -119,6 +119,11 @@ sysexits_h[] =
 #endif /* HAVE_PGP */
 
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif /* HAVE_SMIME */
+
+
 
 #define DISPOSITION(X) X==DISPATTACH?"attachment":"inline"
 
@@ -143,18 +148,6 @@ static void encode_quoted (FGETCONV * fc, FILE *fout, int istext)
 
   while ((c = fgetconv (fc)) != EOF)
   {
-    /* Escape lines that begin with "the message separator". */
-    if (linelen == 5 && !mutt_strncmp ("From ", line, 5))
-    {
-      strfcpy (line, "=46rom ", sizeof (line));
-      linelen = 7;
-    }
-    else if (linelen == 1 && line[0] == '.')
-    {
-      strfcpy (line, "=2E", sizeof (line));
-      linelen = 3;
-    }
-
     /* Wrap the line if needed. */
     if (linelen == 76 && ((istext && c != '\n') || !istext))
     {
@@ -185,6 +178,24 @@ static void encode_quoted (FGETCONV * fc, FILE *fout, int istext)
       }
     }
 
+    /* Escape lines that begin with/only contain "the message separator". */
+    if (linelen == 4 && !mutt_strncmp ("From", line, 4))
+    {
+      strfcpy (line, "=46rom", sizeof (line));
+      linelen = 6;
+    }
+    else if (linelen == 4 && !mutt_strncmp ("from", line, 4))
+    {
+      strfcpy (line, "=66rom", sizeof (line));
+      linelen = 6;
+    }
+    else if (linelen == 1 && line[0] == '.')
+    {
+      strfcpy (line, "=2E", sizeof (line));
+      linelen = 3;
+    }
+
+
     if (c == '\n' && istext)
     {
       /* Check to make sure there is no trailing space on this line. */
@@ -441,6 +452,12 @@ int mutt_write_mime_header (BODY *a, FILE *f)
   return (ferror (f) ? -1 : 0);
 }
 
+#ifdef HAVE_PGP
+# define write_as_text_part(a) (mutt_is_text_part(a) || mutt_is_application_pgp(a))
+#else
+# define write_as_text_part(a) (mutt_is_text_part(a))
+#endif
+
 int mutt_write_mime_body (BODY *a, FILE *f)
 {
   char *p, boundary[SHORT_STRING];
@@ -495,17 +512,17 @@ int mutt_write_mime_body (BODY *a, FILE *f)
 
   if (a->type == TYPETEXT && (!a->noconv))
     fc = fgetconv_open (fpin, Charset, 
-			mutt_get_body_charset (send_charset, sizeof (send_charset), a), 
-			M_ICONV_HOOK_TO);
+			mutt_get_body_charset (send_charset, sizeof (send_charset), a),
+			0);
   else
     fc = fgetconv_open (fpin, 0, 0, 0);
 
   if (a->encoding == ENCQUOTEDPRINTABLE)
-    encode_quoted (fc, f, mutt_is_text_type (a->type, a->subtype));
+    encode_quoted (fc, f, write_as_text_part (a));
   else if (a->encoding == ENCBASE64)
-    encode_base64 (fc, f, mutt_is_text_type (a->type, a->subtype));
+    encode_base64 (fc, f, write_as_text_part (a));
   else if (a->type == TYPETEXT && (!a->noconv))
-    encode_8bit (fc, f, mutt_is_text_type (a->type, a->subtype));
+    encode_8bit (fc, f, write_as_text_part (a));
   else
     mutt_copy_stream (fpin, f);
 
@@ -515,6 +532,8 @@ int mutt_write_mime_body (BODY *a, FILE *f)
   return (ferror (f) ? -1 : 0);
 }
 
+#undef write_as_text_part
+
 #define BOUNDARYLEN 16
 void mutt_generate_boundary (PARAMETER **parm)
 {
@@ -613,7 +632,7 @@ static void update_content_info (CONTENT *info, CONTENT_STATE *s, char *d, size_
     {
       if (linelen == 1)
       {
-        if (ch == 'F')
+        if ((ch == 'F') || (ch == 'f'))
           from = 1;
         else
           from = 0;
@@ -671,9 +690,10 @@ static size_t convert_file_to (FILE *file, const char *fromcode,
 			       int ncodes, const char **tocodes,
 			       int *tocode, CONTENT *info)
 {
+#ifdef HAVE_ICONV
   iconv_t cd1, *cd;
   char bufi[256], bufu[512], bufo[4 * sizeof (bufi)];
-  const char *ib, *ub;
+  ICONV_CONST char *ib, *ub;
   char *ob;
   size_t ibl, obl, ubl, ubl1, n, ret;
   int i;
@@ -681,7 +701,7 @@ static size_t convert_file_to (FILE *file, const char *fromcode,
   CONTENT_STATE *states;
   size_t *score;
 
-  cd1 = mutt_iconv_open ("UTF-8", fromcode, M_ICONV_HOOK_FROM);
+  cd1 = mutt_iconv_open ("UTF-8", fromcode, 0);
   if (cd1 == (iconv_t)(-1))
     return -1;
 
@@ -795,6 +815,9 @@ static size_t convert_file_to (FILE *file, const char *fromcode,
   safe_free ((void **) &states);
 
   return ret;
+#else
+  return -1;
+#endif /* !HAVE_ICONV */
 }
 
 /*
@@ -959,7 +982,7 @@ CONTENT *mutt_get_content_info (const char *fname, BODY *b)
  * exists.
  */
 
-static int lookup_mime_type (BODY *att, const char *path)
+int mutt_lookup_mime_type (BODY *att, const char *path)
 {
   FILE *f;
   char *p, *q, *ct;
@@ -994,7 +1017,7 @@ static int lookup_mime_type (BODY *att, const char *path)
 	strfcpy (buf, PKGDATADIR"/mime.types", sizeof (buf));
 	break;
       default:
-	dprint (1, (debugfile, "lookup_mime_type: Internal error, count = %d.\n", count));
+        dprint (1, (debugfile, "mutt_lookup_mime_type: Internal error, count = %d.\n", count));
 	goto bye;	/* shouldn't happen */
     }
 
@@ -1192,7 +1215,7 @@ static void mutt_set_encoding (BODY *b, CONTENT *info)
   if (b->type == TYPETEXT)
   {
     char *chsname = mutt_get_body_charset (send_charset, sizeof (send_charset), b);
-    if ((info->lobin && strncasecmp (chsname, "iso-2022", 8)) || info->linemax > 990 || (info->from && option (OPTENCODEFROM)))
+    if ((info->lobin && ascii_strncasecmp (chsname, "iso-2022", 8)) || info->linemax > 990 || (info->from && option (OPTENCODEFROM)))
       b->encoding = ENCQUOTEDPRINTABLE;
     else if (info->hibin)
       b->encoding = option (OPTALLOW8BIT) ? ENC8BIT : ENCQUOTEDPRINTABLE;
@@ -1211,6 +1234,8 @@ static void mutt_set_encoding (BODY *b, CONTENT *info)
     else
       b->encoding = ENC7BIT;
   }
+  else if (b->type == TYPEAPPLICATION && ascii_strcasecmp (b->subtype, "pgp-keys") == 0)
+    b->encoding = ENC7BIT;
   else
 #if 0
     if (info->lobin || info->hibin || info->binary || info->linemax > 990
@@ -1286,15 +1311,17 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   BODY *body;
   FILE *fp;
   int cmflags, chflags;
-#ifdef HAVE_PGP
-  int pgp = hdr->pgp;
+#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
+  int pgp = hdr->security;
 #endif
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
   if ((option(OPTMIMEFORWDECODE) || option(OPTFORWDECRYPT)) &&
-      (hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
-    return (NULL);
-#endif /* HAVE_PGP */
+      (hdr->security & ENCRYPT)) {
+    if (!crypt_valid_passphrase(hdr->security))
+      return (NULL);
+  }
+#endif /* defined(HAVE_PGP) ||  defined(HAVE_SMIME) */
 
   mutt_mktemp (buffer);
   if ((fp = safe_fopen (buffer, "w+")) == NULL)
@@ -1320,13 +1347,17 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
     cmflags = M_CM_DECODE | M_CM_CHARCONV;
 #ifdef HAVE_PGP
     pgp &= ~PGPENCRYPT;
+#endif
+#ifdef HAVE_SMIME
+    pgp &= ~SMIMEENCRYPT;
 #endif
   }
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
   else
     if (option (OPTFORWDECRYPT)
-       && (hdr->pgp & PGPENCRYPT))
+       && (hdr->security & ENCRYPT))
   {
+#ifdef HAVE_PGP
     if (mutt_is_multipart_encrypted (hdr->content))
     {
       chflags |= CH_MIME | CH_NONEWLINE;
@@ -1339,6 +1370,18 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
       cmflags = M_CM_DECODE | M_CM_CHARCONV;
       pgp &= ~PGPENCRYPT;
     }
+#endif
+#if defined(HAVE_PGP) &&  defined(HAVE_SMIME)
+    else
+#endif
+#ifdef HAVE_SMIME
+    if (mutt_is_application_smime (hdr->content) & SMIMEENCRYPT)
+    {
+      chflags |= CH_MIME | CH_TXTPLAIN;
+      cmflags = M_CM_DECODE | M_CM_CHARCONV;
+      pgp &= ~SMIMEENCRYPT;
+    }
+#endif
   }
 #endif
 
@@ -1351,8 +1394,8 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   body->hdr->offset = 0;
   /* we don't need the user headers here */
   body->hdr->env = mutt_read_rfc822_header(fp, body->hdr, 0, 0);
-#ifdef HAVE_PGP
-  body->hdr->pgp = pgp;
+#if defined(HAVE_PGP) ||  defined(HAVE_SMIME)
+  body->hdr->security = pgp;
 #endif /* HAVE_PGP */
   mutt_update_encoding (body);
   body->parts = body->hdr->content;
@@ -1376,7 +1419,7 @@ BODY *mutt_make_file_attach (const char *path)
 
 #if 0
   
-  if ((n = lookup_mime_type (buf, sizeof (buf), xbuf, sizeof (xbuf), path)) != TYPEOTHER 
+  if ((n = mutt_lookup_mime_type (buf, sizeof (buf), xbuf, sizeof (xbuf), path)) != TYPEOTHER 
       || *xbuf != '\0')
   {
     att->type = n;
@@ -1386,7 +1429,7 @@ BODY *mutt_make_file_attach (const char *path)
 
 #else
   
-  lookup_mime_type (att, path);
+  mutt_lookup_mime_type (att, path);
 
 #endif
   
@@ -1574,6 +1617,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   char buffer[LONG_STRING];
   char *p;
   LIST *tmp = env->userhdrs;
+  int has_agent = 0; /* user defined user-agent header field exists */
   
   if (mode == 0 && !privacy)
     fputs (mutt_make_date (buffer, sizeof(buffer)), fp);
@@ -1659,12 +1703,6 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
     fputc ('\n', fp);
   }
   
-  if (mode == 0 && !privacy && option (OPTXMAILER))
-  {
-    /* Add a vanity header */
-    fprintf (fp, "User-Agent: Mutt/%s\n", MUTT_VERSION);
-  }
-
   /* Add any user defined headers */
   for (; tmp; tmp = tmp->next)
   {
@@ -1673,11 +1711,25 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
       p++; SKIPWS (p);
       if (!*p) 	continue;  /* don't emit empty fields. */
 
+      /* check to see if the user has overridden the user-agent field */
+      if (!ascii_strncasecmp ("user-agent", tmp->data, 10))
+      {
+	has_agent = 1;
+	if (privacy)
+	  continue;
+      }
+
       fputs (tmp->data, fp);
       fputc ('\n', fp);
     }
   }
 
+  if (mode == 0 && !privacy && option (OPTXMAILER) && !has_agent)
+  {
+    /* Add a vanity header */
+    fprintf (fp, "User-Agent: Mutt/%s\n", MUTT_VERSION);
+  }
+
   return (ferror (fp) == 0 ? 0 : -1);
 }
 
@@ -2167,58 +2219,64 @@ void mutt_unprepare_envelope (ENVELOPE *env)
   rfc2047_decode (&env->subject);
 }
 
-static void _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *resent_from,
+static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *resent_from,
 				  ADDRESS *env_from)
 {
-  int i;
+  int i, ret = 0;
   FILE *f;
   char date[SHORT_STRING], tempfile[_POSIX_PATH_MAX];
   MESSAGE *msg = NULL;
 
   if (!h)
   {
+	  /* Try to bounce each message out, aborting if we get any failures. */
     for (i=0; i<Context->msgcount; i++)
       if (Context->hdrs[i]->tagged)
-	_mutt_bounce_message (fp, Context->hdrs[i], to, resent_from, env_from);
-    return;
+        ret |= _mutt_bounce_message (fp, Context->hdrs[i], to, resent_from, env_from);
+    return ret;
   }
 
+  /* If we failed to open a message, return with error */
   if (!fp && (msg = mx_open_message (Context, h->msgno)) == NULL)
-    return;
+    return -1;
 
   if (!fp) fp = msg->fp;
 
   mutt_mktemp (tempfile);
   if ((f = safe_fopen (tempfile, "w")) != NULL)
   {
-    int ch_flags = CH_XMIT | CH_NONEWLINE;
+    int ch_flags = CH_XMIT | CH_NONEWLINE | CH_NOQFROM;
     
     if (!option (OPTBOUNCEDELIVERED))
       ch_flags |= CH_WEED_DELIVERED;
     
     fseek (fp, h->offset, 0);
-    mutt_copy_header (fp, h, f, ch_flags, NULL);
     fprintf (f, "Resent-From: %s", resent_from);
     fprintf (f, "\nResent-%s", mutt_make_date (date, sizeof(date)));
+    fprintf (f, "Resent-Message-ID: %s\n", mutt_gen_msgid());
     fputs ("Resent-To: ", f);
     mutt_write_address_list (to, f, 11);
+    mutt_copy_header (fp, h, f, ch_flags, NULL);
     fputc ('\n', f);
     mutt_copy_bytes (fp, f, h->content->length);
     fclose (f);
 
-    mutt_invoke_sendmail (env_from, to, NULL, NULL, tempfile,
-			  h->content->encoding == ENC8BIT);
+    ret = mutt_invoke_sendmail (env_from, to, NULL, NULL, tempfile,
+			  	h->content->encoding == ENC8BIT);
   }
 
   if (msg)
     mx_close_message (&msg);
+
+  return ret;
 }
 
-void mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
+int mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
 {
   ADDRESS *from;
   const char *fqdn = mutt_fqdn (1);
   char resent_from[STRING];
+  int ret;
 
   resent_from[0] = '\0';
   from = mutt_default_from ();
@@ -2230,9 +2288,11 @@ void mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to)
   
   rfc822_write_address (resent_from, sizeof (resent_from), from);
 
-  _mutt_bounce_message (fp, h, to, resent_from, from);
+  ret = _mutt_bounce_message (fp, h, to, resent_from, from);
 
   rfc822_free_address (&from);
+
+  return ret;
 }
 
 
@@ -2356,12 +2416,12 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
 
 #ifdef HAVE_PGP
   /* (postponment) if the mail is to be signed or encrypted, save this info */
-  if (post && (hdr->pgp & (PGPENCRYPT | PGPSIGN)))
+  if (post && (hdr->security & APPLICATION_PGP))
   {
     fputs ("X-Mutt-PGP: ", msg->fp);
-    if (hdr->pgp & PGPENCRYPT) 
+    if (hdr->security & ENCRYPT) 
       fputc ('E', msg->fp);
-    if (hdr->pgp & PGPSIGN)
+    if (hdr->security & SIGN)
     {
       fputc ('S', msg->fp);
       if (PgpSignAs && *PgpSignAs)
@@ -2371,6 +2431,25 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
   }
 #endif /* HAVE_PGP */
 
+#ifdef HAVE_SMIME
+  /* (postponment) if the mail is to be signed or encrypted, save this info */
+  if (post && (hdr->security & APPLICATION_SMIME))
+  {
+    fputs ("X-Mutt-SMIME: ", msg->fp);
+    if (hdr->security & ENCRYPT) {
+	fputc ('E', msg->fp);
+	if (SmimeCryptAlg && *SmimeCryptAlg)
+	    fprintf (msg->fp, "C<%s>", SmimeCryptAlg);
+    }
+    if (hdr->security & SIGN) {
+	fputc ('S', msg->fp);
+	if (SmimeDefaultKey && *SmimeDefaultKey)
+	    fprintf (msg->fp, "<%s>", SmimeDefaultKey);
+    }
+    fputc ('\n', msg->fp);
+  }
+#endif /* HAVE_SMIME */
+
 #ifdef MIXMASTER
   /* (postponement) if the mail is to be sent through a mixmaster 
    * chain, save that information
